package com.phantomsxr.armodplugin

import android.util.Log
class ARMODCallbackAPI {
    companion object{
        var instance:ARMODCallbackAPI? = null
    }

    init {
        instance = this
        deviceNotSupport()
    }
    /**
     *[Android Only]When your device supports ARCore but you are installing the ARCore service,
     *this method will be executed, prompting the user to install the ARCore service.
     */
    fun deviceNotSupport() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onDeviceNotSupport()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Start loading AR experience.
     */
    fun addLoadingOverlay() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onAddLoadingOverlay()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Loading AR experience asset progress.
     *[progress] loading progress value(0.0~1.0).
     */
    fun updateLoadingProgress(progress:Float) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onUpdateLoadingProgress(progress)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Remove loading overlay and loading AR experience finished
     */
    fun removeLoadingOverlay() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRemoveLoadingOverlay()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }
    /**
     * The method will be executed if an exception occurs.
     * [errorMsg] error message
     * [errorCode] the error id
     */
    fun  throwException(errorMsg:String,errorCode:Int) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onThrowException(errorMsg,errorCode)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * This device is support AR-MOD but need to install ARCore services.
     */
    fun  needInstallARCoreService() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onNeedInstallARCoreService()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * After the AR algorithm is initialized, the method will be executed
     */
    fun  sdkInitialized() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onSdkInitialized()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Open the url in builtin browser from AR experience
     */
    fun  openBuiltInBrowser(url:String) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onOpenBuiltInBrowser(url)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Start image recognition[WIP]
     */
    fun  recognitionStart() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRecognitionStart()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Finished image recognition[WIP]
     */
    fun  recognitionComplete() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRecognitionComplete()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Get any information from app to AR experiences
     * [opTag] operation type tag
     * [callback] return data
     */
    fun tryAcquireInformation(opTag:String,callback:AndroidCallback) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onTryAcquireInformation(opTag,callback)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Detected that the AR experience package is too large
     * [currentSize] Current AR experience package size
     * [presetSize] Maximum downloadable package size
     */
    fun packageSizeMoreThanPresetSize(currentSize:String,presetSize:String) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onPackageSizeMoreThanPresetSize(currentSize,presetSize)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     *AR-MOD SDK Exited
     */
    fun onARMODExit() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onARMODExit()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * AR-MOD SDK Start
     */
    fun onARMODLaunch() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onARMODLaunch()
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }
}