package com.phantomsxr.armodplugin

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.WindowManager
import java.util.Objects

import com.unity3d.player.UnityPlayerActivity;

class ARMODNativeActivity : UnityPlayerActivity() {
    private lateinit var mMainActivityClass: Class<*>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        instance = this
        this.window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
        val intent = intent
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
            handleIntent(intent)
        }
    }

    private fun unloadPlayer() {
        mUnityPlayer?.unload()
        showMainActivity()
    }

    private fun quitPlayer() {
        mUnityPlayer?.quit()
    }

    fun showMainActivity() {
        val intent = Intent(this, mMainActivityClass)
        intent.putExtra("showMain", true)
        intent.flags = Intent.FLAG_ACTIVITY_REORDER_TO_FRONT or Intent.FLAG_ACTIVITY_SINGLE_TOP
        startActivity(intent)
    }

    override fun onUnityPlayerUnloaded() {
        showMainActivity()
    }

    override fun onLowMemory() {
        super.onLowMemory()
        mUnityPlayer?.lowMemory()
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
            handleIntent(intent)
        }
        setIntent(intent)
    }

    private fun handleIntent(intent: Intent) {
        // Set activity not fullscreen
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
            val st = Objects.requireNonNull(intent.extras)?.get("flutterActivity") as Class<*>
            if (st != null) {
                mMainActivityClass = st
            }
            // Set activity not fulllscreen
            if (Objects.requireNonNull(intent.extras)?.getBoolean("fullscreen") == true) {
                val fullscreen = intent.extras?.getBoolean("fullscreen")
                if (!fullscreen!!) {
                    this.window.addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN)
                    this.window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)
                } else {
                    this.window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
                }
            }
            // Unloads armod
            if (Objects.requireNonNull(intent.extras)?.containsKey("unload") == true) {
                mUnityPlayer?.unload()
            }
        }
    }

    override fun onBackPressed() {
        Log.i(LOG_TAG, "onBackPressed called")
        this.showMainActivity()
        super.onBackPressed()
    }

    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
    }

    override fun onPause() {
        super.onPause()
        this.mUnityPlayer?.pause()
    }

    override fun onResume() {
        super.onResume()
        this.mUnityPlayer?.resume()
    }

    override fun onDestroy() {
        super.onDestroy()
        instance = null
    }

    companion object {
        var instance: ARMODNativeActivity? = null
        internal val LOG_TAG = "OverrideARMODActivity"
    }

    /**
     *[Android Only]When your device supports ARCore but you are installing the ARCore service,
     *this method will be executed, prompting the user to install the ARCore service.
     */
    fun onDeviceNotSupport() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onDeviceNotSupport()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Start loading AR experience.
     */
    fun onAddLoadingOverlay() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onAddLoadingOverlay()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Loading AR experience asset progress.
     *[progress] loading progress value(0.0~1.0).
     */
    fun onUpdateLoadingProgress(progress:Float) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onUpdateLoadingProgress(progress)
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Remove loading overlay and loading AR experience finished
     */
    fun onRemoveLoadingOverlay() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRemoveLoadingOverlay()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }
    /**
     * The method will be executed if an exception occurs.
     * [errorMsg] error message
     * [errorCode] the error id
     */
    fun  onThrowException(errorMsg:String,errorCode:Int) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onThrowException(errorMsg,errorCode)
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    fun  onNeedInstallARCoreService() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onNeedInstallARCoreService()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * After the AR algorithm is initialized, the method will be executed
     */
    fun  onSdkInitialized() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onSdkInitialized()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Open the url in builtin browser from AR experience
     */
    fun  onOpenBuiltInBrowser(url:String) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onOpenBuiltInBrowser(url)
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Start image recognition[WIP]
     */
    fun  onRecognitionStart() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRecognitionStart()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Finished image recognition[WIP]
     */
    fun  onRecognitionComplete() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onRecognitionComplete()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * Get any information from app to AR experiences
     * [opTag] operation type tag
     * [callback] return data
     */
    fun  tryAcquireInformation(opTag:String,callback:AndroidCallback) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onTryAcquireInformation(opTag,callback)
            } catch (e: Exception) {
                e.message?.let { Log.e(ARMODNativeActivity.LOG_TAG, it) }
            }
        }
    }

    /**
     * Detected that the AR experience package is too large
     * [currentSize] Current AR experience package size
     * [presetSize] Maximum downloadable package size
     */
    fun onPackageSizeMoreThanPresetSize(currentSize:String,presetSize:String) {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onPackageSizeMoreThanPresetSize(currentSize,presetSize)
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     *AR-MOD SDK Exited
     */
    fun onARMODExit() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onARMODExit()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }

    /**
     * AR-MOD SDK Start
     */
    fun onARMODLaunch() {
        for (listener in ARMODPlayerUtils.mARMODEventListeners) {
            try {
                listener.onARMODLaunch()
            } catch (e: Exception) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
    }
}