//
//  ARMODPlayerUtils.swift
//  flutter_armod_widget
//
//  Created by PhantomsXR on 23/08/2021.
//

import Foundation
import UnityFramework

private var armod_warmed_up = false
// Hack to work around iOS SDK 4.3 linker problem
// we need at least one __TEXT, __const section entry in main application .o files
// to get this section emitted at right time and so avoid LC_ENCRYPTION_INFO size miscalculation
private let constsection = 0

// keep arg for armod init from non main
var gArgc: Int32 = 0
var gArgv: UnsafeMutablePointer<UnsafeMutablePointer<Int8>?>? = nil
var appLaunchOpts: [UIApplication.LaunchOptionsKey: Any]? = [:]

/***********************************PLUGIN_ENTRY STARTS**************************************/
public func InitARMODIntegration(argc: Int32, argv: UnsafeMutablePointer<UnsafeMutablePointer<Int8>?>?) {
    gArgc = argc
    gArgv = argv
}

public func InitARMODIntegrationWithOptions(
    argc: Int32,
    argv: UnsafeMutablePointer<UnsafeMutablePointer<Int8>?>?,
    _ launchingOptions:  [UIApplication.LaunchOptionsKey: Any]?) {
    gArgc = argc
    gArgv = argv
    appLaunchOpts = launchingOptions
}
/***********************************PLUGIN_ENTRY END**************************************/

// Load armod framework for fisrt run
func UnityFrameworkLoad() -> UnityFramework? {
    var bundlePath: String? = nil
    bundlePath = Bundle.main.bundlePath
    bundlePath = (bundlePath ?? "") + "/Frameworks/UnityFramework.framework"
    
    let bundle = Bundle(path: bundlePath ?? "")
    if bundle?.isLoaded == false {
        bundle?.load()
    }
    
    return bundle?.principalClass?.getInstance()
}

/*********************************** GLOBAL FUNCS & VARS START**************************************/

public var globalChannel: FlutterMethodChannel? = nil
private var armodPlayerUtils: ARMODPlayerUtils? = nil
public func GetARMODPlayerUtils() -> ARMODPlayerUtils? {
    
    if armodPlayerUtils == nil {
        armodPlayerUtils = ARMODPlayerUtils()
    }
    return armodPlayerUtils
}

/*********************************** GLOBAL FUNCS & VARS END****************************************/

var controller: UnityAppController?
var sharedApplication: UIApplication?

@objc protocol ARMODEventListener: AnyObject {
    
    func onReceiveMessage(_ message: UnsafePointer<Int8>?)
    
}

@objc public class ARMODPlayerUtils: UIResponder, UIApplicationDelegate, UnityFrameworkListener {
    
    var ufw: UnityFramework!
    private var _isARMODPaused = false
    private var _isARMODReady = false
    private var _isARMODLoaded = false
    
    func initARMOD() {
        if (self.armodIsInitiallized()) {
            self.ufw?.showUnityWindow()
            return
        }
        
        self.ufw = UnityFrameworkLoad()
        
        self.ufw?.setDataBundleId("com.unity3d.framework")
        
        registerARMODListener()
        self.ufw?.runEmbedded(withArgc: gArgc, argv: gArgv, appLaunchOpts: appLaunchOpts)
        
        if self.ufw?.appController() != nil {
            controller = self.ufw?.appController()
            self.ufw?.appController()?.window?.windowLevel = UIWindow.Level(UIWindow.Level.normal.rawValue - 1)
        }
        _isARMODLoaded = true
    }
    
    // check if armod is initiallized
    func armodIsInitiallized() -> Bool {
        if self.ufw != nil {
            return true
        }
        
        return false
    }
    
    // Create new armod player
    func createPlayer(completed: @escaping (_ view: UIView?) -> Void) {
        if self.armodIsInitiallized() && self._isARMODReady {
            completed(controller?.rootView)
        }
        
        NotificationCenter.default.addObserver(forName: NSNotification.Name("ARMODReady"), object: nil, queue: OperationQueue.main, using: { note in
            self._isARMODReady = true
            completed(controller?.rootView)
        })
        
        DispatchQueue.main.async {
            if (sharedApplication == nil) {
                sharedApplication = UIApplication.shared
            }
            
            // Always keep Flutter window on top
            let flutterUIWindow = sharedApplication?.keyWindow
            flutterUIWindow?.windowLevel = UIWindow.Level(UIWindow.Level.normal.rawValue + 1) // Always keep Flutter window in top
            sharedApplication?.keyWindow?.windowLevel = UIWindow.Level(UIWindow.Level.normal.rawValue + 1)
            
            self.initARMOD()
            armod_warmed_up = true
            self.listenAppState()
        }
        
        if armod_warmed_up == true {
            self._isARMODReady = true
            self._isARMODLoaded = true
            completed(controller?.rootView)
        }
    }
    
    func registerARMODListener() {
        if self.armodIsInitiallized() {
            self.ufw?.register(self)
        }
    }
    
    func unregisterARMODListener() {
        if self.armodIsInitiallized() {
            self.ufw?.unregisterFrameworkListener(self)
        }
    }
    
    @objc
    public func unityDidUnload(_ notification: Notification!) {
        
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name("ARMODReady"), object: nil)
        if(self.ufw != nil){
            self.unregisterARMODListener()
            self.ufw = nil
        }
        
        self._isARMODReady = false
        self._isARMODLoaded = false
    }
    
    @objc func handleAppStateDidChange(notification: Notification?) {
        if !self._isARMODReady {
            return
        }
        
        let armodAppController = self.ufw?.appController as? UnityAppController
        let application = UIApplication.shared
        
        if notification?.name == UIApplication.willResignActiveNotification {
            armodAppController?.applicationWillResignActive(application)
        } else if notification?.name == UIApplication.didEnterBackgroundNotification {
            armodAppController?.applicationDidEnterBackground(application)
        } else if notification?.name == UIApplication.willEnterForegroundNotification {
            armodAppController?.applicationWillEnterForeground(application)
        } else if notification?.name == UIApplication.didBecomeActiveNotification {
            armodAppController?.applicationDidBecomeActive(application)
        } else if notification?.name == UIApplication.willTerminateNotification {
            armodAppController?.applicationWillTerminate(application)
        } else if notification?.name == UIApplication.didReceiveMemoryWarningNotification {
            armodAppController?.applicationDidReceiveMemoryWarning(application)
        }
    }
    
    
    // Listener for app lifecycle eventa
    func listenAppState() {
        for name in [
            UIApplication.didBecomeActiveNotification,
            UIApplication.didEnterBackgroundNotification,
            UIApplication.willTerminateNotification,
            UIApplication.willResignActiveNotification,
            UIApplication.willEnterForegroundNotification,
            UIApplication.didReceiveMemoryWarningNotification
        ] {
            guard let name = name as? String else {
                continue
            }
            NotificationCenter.default.addObserver(
                self,
                selector: #selector(self.handleAppStateDidChange),
                name: NSNotification.Name(name),
                object: nil)
        }
    }
    // Pause armod player
    func pause() {
        self.ufw?.pause(true)
        self._isARMODPaused = true
    }
    
    // Resume armod player
    func resume() {
        self.ufw?.pause(false)
        self._isARMODPaused = false
    }
    
    // Unoad armod player
    func unload() {
        self.ufw?.unloadApplication()
    }
    
    public  func isARMODLoaded() -> Bool {
        return _isARMODLoaded
    }
    
    public func isARMODPaused() -> Bool {
        return _isARMODPaused
    }
    
    public func isARMODReady()->Bool{
        return self._isARMODReady
    }
    
    // Quit armod player application
    func quit() {
        self.ufw?.quitApplication(0)
        self._isARMODLoaded = false
    }
    
    // Post message to armod
    func postMessageToARMOD(gameObject: String?, unityMethodName: String?, unityMessage: String?) {
        if self.armodIsInitiallized() {
            self.ufw?.sendMessageToGO(withName: gameObject, functionName: unityMethodName, message: unityMessage)
        }
    }
}
