//
//  FLTARMODWidgetController.swift
//  flutter_armod_widget
//
//  Created by PhantomsXR on 23/08/2021.
//

import Foundation
import UnityFramework

// Defines armod controllable from Flutter.
class FLTARMODWidgetController: NSObject, FLTARMODOptionsSink, FlutterPlatformView {
    private var fltARMODView: FLTARMODView
    private var viewId: Int64 = 0
    private var channel: FlutterMethodChannel?
    private weak var registrar: (NSObjectProtocol & FlutterPluginRegistrar)?
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        registrar: NSObjectProtocol & FlutterPluginRegistrar
    ) {
        self.fltARMODView = FLTARMODView(frame: frame)
        super.init()
        
        self.viewId = viewId
        
        let channelName = String(format: "plugin.phantomsxr.com/armod_view_%lld", viewId)
        self.channel = FlutterMethodChannel(name: channelName, binaryMessenger: registrar.messenger())
        globalChannel = self.channel
        
        self.channel?.setMethodCallHandler(self.methodHandler)
        self.initView()
    }
    
    func methodHandler(_ call: FlutterMethodCall, result: FlutterResult) {
        if call.method == "armod#dispose" {
            self.dispose()
            result(nil)
        } else {
            self.reattachView()
            if call.method == "armod#isReady" {
                result(GetARMODPlayerUtils()?.armodIsInitiallized())
            } else if call.method == "armod#isLoaded" {
                let _isUnloaded = GetARMODPlayerUtils()?.isARMODLoaded()
                result(_isUnloaded)
            } else if call.method == "armod#createARMOD" {
                self.initView()
                result(nil)
            } else if call.method == "armod#isPaused" {
                let _isPaused = GetARMODPlayerUtils()?.isARMODPaused()
                result(_isPaused)
            } else if call.method == "armod#pausePlayer" {
                GetARMODPlayerUtils()?.pause()
                result(nil)
            } else if call.method == "armod#postMessage" {
                self.postMessage(call: call, result: result)
                result(nil)
            } else if call.method == "armod#resumePlayer" {
                GetARMODPlayerUtils()?.resume()
                result(nil)
            } else if call.method == "armod#unloadPlayer" {
                GetARMODPlayerUtils()?.unload()
                result(nil)
            } else if call.method == "armod#quitPlayer" {
                GetARMODPlayerUtils()?.quit()
                result(nil)
            } else if call.method == "armod#closeARMOD" {
                GetARMODPlayerUtils()?.postMessageToARMOD(gameObject: "EntryPoint", unityMethodName: "Dispose", unityMessage: "")
                result(nil)
            }
            
            else if call.method == "armod#setDevicesOrientation" {
                
                guard let args = call.arguments else {
                    result("iOS could not recognize flutter arguments in method: (armod#setDevicesOrientation)")
                    return
                }
                
                if let myArgs = args as? [String: Any],
                    let message = myArgs["message"] as? String {
                    GetARMODPlayerUtils()?.postMessageToARMOD(gameObject: "EntryPoint", unityMethodName: "SetUIInterfaceOrientation", unityMessage: message)
                    result(nil)
                }
                
            }  else if call.method == "armod#waitForARMOD" {
                result(nil)
            } else {
                result(FlutterMethodNotImplemented)
            }
        }
    }
    
    func setDisabledUnload(enabled: Bool) {
        
    }
    
    func view() -> UIView {
        return fltARMODView
    }
    
    func initView() {
        if (GetARMODPlayerUtils()?.armodIsInitiallized() == true) {
            fltARMODView.setARMODView(GetARMODPlayerUtils()?.ufw?.appController()?.rootView)
        } else {
            GetARMODPlayerUtils()?.createPlayer(completed: { [self] (view: UIView?) in
                if let v = view {
                    fltARMODView.setARMODView(v)
                    self.channel?.invokeMethod("events#onARMODCreated", arguments: nil)
                }
            })
        }
    }
    
    func attachView() {
        if (GetARMODPlayerUtils() != nil) {
            GetARMODPlayerUtils()?.initARMOD()
            let armodView = GetARMODPlayerUtils()?.ufw?.appController()?.rootView
            if let superview = armodView?.superview {
                armodView?.removeFromSuperview()
                superview.layoutIfNeeded()
            }
            
            if let armodView = armodView {
                fltARMODView.addSubview(armodView)
            }
            GetARMODPlayerUtils()?.resume()
        }
    }
    
    func reattachView() {
        let armodView = GetARMODPlayerUtils()?.ufw?.appController()?.rootView
        let superview = armodView?.superview
        if superview != fltARMODView {
            attachView()
        }
    }
    
    func dispose() {
        channel?.setMethodCallHandler(nil)
        globalChannel?.setMethodCallHandler(nil)
        if GetARMODPlayerUtils() != nil {
            let armodView = GetARMODPlayerUtils()?.ufw?.appController()?.rootView
            let superview = armodView?.superview
            armodView?.removeFromSuperview()
            superview?.layoutIfNeeded()
            GetARMODPlayerUtils()?.pause()
        }
    }
    
    func postMessage(call: FlutterMethodCall, result: FlutterResult) {
        guard let args = call.arguments else {
            result("iOS could not recognize flutter arguments in method: (postMessage)")
            return
        }
        
        if let myArgs = args as? [String: Any],
           let gObj = myArgs["gameObject"] as? String,
           let method = myArgs["methodName"] as? String,
           let message = myArgs["message"] as? String {
            GetARMODPlayerUtils()?.postMessageToARMOD(gameObject: gObj, unityMethodName: method, unityMessage: message)
            result(nil)
        } else {
            result(FlutterError(code: "-1", message: "iOS could not extract " +
                                "flutter arguments in method: (postMessage)", details: nil))
        }
    }
}
