part of flutter_armod_widget;

class ARMODWidget extends StatefulWidget {
  ///Event fires when the armod player is created.
  final ARMODCreatedCallback onARMODCreated;

  ///Current device is not support ARMOD
  final DeviceNotSupportCallback? onDeviceNotSupport;

  ///Start loading AR experience
  final AddLoadingOverlayCallback? onAddLoadingOverlay;

  final RemoveLoadingOverlayCallback? onRemoveLoadingOverlay;

  ///Loading AR experience asset progress
  ///[progress] loading progress value(0.0~1.0).
  final UpdateLoadingProgressCallback? onUpdateLoadingProgress;

  ///The method will be executed if an exception occurs
  ///[message] error message
  ///[errorCode] the error id
  final ThrowExceptionCallback? onThrowException;

  ///[Android Only]When your device supports ARCore but you are installing the ARCore service,
  ///this method will be executed, prompting the user to install the ARCore service.
  final NeedInstallARCoreServiceCallback? onNeedInstallARCoreService;

  ///After the AR algorithm is initialized, the method will be executed
  final SdkInitializedCallback? onSdkInitialized;

  ///Open the url in builtin browser from AR experience
  final OpenBuiltInBrowserCallback? onOpenBuiltInBrowser;

  ///Start image recognition[WIP]
  final RecognitionStartCallback? onRecognitionStart;

  ///Finished image recognition[WIP]
  final RecognitionCompleteCallback? onRecognitionComplete;

  ///Get any information from app to AR experiences
  final TryAcquireInformationCallback? onTryAcquireInformation;

  ///Detected that the AR experience package is too large
  final PackageSizeMoreThanPresetSizeCallback? onPackageSizeMoreThanPresetSize;

  ///AR-MOD SDK Exited
  final ARMODExitCallback? onARMODExit;

  ///AR-MOD SDK Start
  final ARMODLaunchCallback? onARMODLaunch;

  final Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers;

  /// Set to true to force armod to fullscreen
  final bool fullscreen;

  /// This flag enables placeholder widget
  final bool enablePlaceholder;

  /// This flag allows you use AndroidView instead of PlatformViewLink for android
  final bool useAndroidView;

  /// This is just a helper to render a placeholder widget
  final Widget? placeholder;

  ARMODWidget({
    Key? key,
    required this.onARMODCreated,
    this.onDeviceNotSupport,
    this.onAddLoadingOverlay,
    this.onRemoveLoadingOverlay,
    this.onUpdateLoadingProgress,
    this.onThrowException,
    this.onNeedInstallARCoreService,
    this.onSdkInitialized,
    this.onOpenBuiltInBrowser,
    this.onRecognitionComplete,
    this.onTryAcquireInformation,
    this.onPackageSizeMoreThanPresetSize,
    this.onARMODLaunch,
    this.onARMODExit,
    this.fullscreen = false,
    this.enablePlaceholder = false,
    this.gestureRecognizers,
    this.placeholder,
    this.useAndroidView = false,
    this.onRecognitionStart,
  });

  @override
  _ARMODWidgetState createState() => _ARMODWidgetState();
}

class _ARMODWidgetState extends State<ARMODWidget> {
  final Completer<ARMODWidgetController> _controller =
      Completer<ARMODWidgetController>();

  @override
  void initState() {
    super.initState();
  }

  @override
  Future<void> dispose() async {
    super.dispose();
    ARMODWidgetController controller = await _controller.future;

    controller.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final Map<String, dynamic> creationParams = <String, dynamic>{
      'fullscreen': widget.fullscreen,
    };

    if (widget.enablePlaceholder) {
      return widget.placeholder ??
          Text('Placeholder mode enabled, no native code will be called');
    }

    return ARMODViewFlutterPlatform.instance.buildView(
        creationParams,
        widget.gestureRecognizers,
        onPlatformViewCreated,
        widget.useAndroidView);
  }

  Future<void> onPlatformViewCreated(int id) async {
    final controller = await ARMODWidgetController.init(id, this);
    _controller.complete(controller);
    final ARMODCreatedCallback? onARMODCreated = widget.onARMODCreated;
    if (onARMODCreated != null) {
      onARMODCreated(controller);
    }
    print('*********************************************');
    print('** flutter armod controller setup complete **');
    print('*********************************************');
  }
}
