part of flutter_armod_widget;

abstract class ARMODViewFlutterPlatform extends PlatformInterface {
  static final Object _token = Object();
  dynamic resultData;

  // Constructs a ARMODViewFlutterPlatform.
  ARMODViewFlutterPlatform() : super(token: _token);

  static ARMODViewFlutterPlatform _instance = MethodChannelARMODViewFlutter();

  //The default instance of [ARMODViewFlutterPlatform] to use.
  //
  // Defaults to [MethodChannelARMODViewFlutter].
  static ARMODViewFlutterPlatform get instance => _instance;

  // Platform-specific plugins should set this with their own platform-specific
  // class that extends [ARMODViewFlutterPlatform] when they register themselves.
  static set instance(ARMODViewFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<void> init(int id) {
    throw UnimplementedError("init() has not been implemented.");
  }

  Future<bool?> isReady({required int id}) async {
    throw UnimplementedError("isReady() has not been implemented.");
  }

  Future<bool?> isPaused({required int id}) async {
    throw UnimplementedError("isPaused() has not been implemented.");
  }

  Future<bool?> isLoaded({required int id}) async {
    throw UnimplementedError("isLoaded() has not been implemented.");
  }

  Future<bool?> inBackground({required int id}) async {
    throw UnimplementedError("inBackground() has not been implemented.");
  }

  Future<bool?> createARMODPlayer({required int id}) async {
    throw UnimplementedError("createARMODPlayer() has not been implemented.");
  }

  Future<bool?> closeARMODPlayer({required int id}) async {
    throw UnimplementedError("createARMODPlayer() has not been implemented.");
  }

  Future<void> setDevicesOrientation(
      {required int id, required String orientation}) async {
    throw UnimplementedError("createARMODPlayer() has not been implemented.");
  }

  Future<void> pausePlayer({required int id}) async {
    throw UnimplementedError('pausePlayer() has not been implemented.');
  }

  Future<void> resumePlayer({required int id}) async {
    throw UnimplementedError('resumePlayer() has not been implemented.');
  }

  Future<void> postMessage(
      {required int id,
      required String gameObject,
      required String methodName,
      required String message}) {
    throw UnimplementedError('postMessage() has not been implemented.');
  }

  // Opens AR-MOD in it's own activity. Android only.
  Future<void> openInNativeProcess({required int id}) async {
    throw UnimplementedError('openInNativeProcess() has not been implemented.');
  }

  Future<void> unloadPlayer({required int id}) async {
    throw UnimplementedError('unloadPlayer() has not been implemented.');
  }

  Future<void> quitPlayer({required int id}) async {
    throw UnimplementedError('quitPlayer() has not been implemented.');
  }

  //ARMOD events
  Stream<deviceNotSupport> onDeviceNotSupport({required int id}) {
    throw UnimplementedError('onDeviceNotSupport() has not been implemented.');
  }

  Stream<removeLoadingOverlay> onRemoveLoadingOverlay({required int id}) {
    throw UnimplementedError(
        'onRemoveLoadingOverlay() has not been implemented.');
  }

  Stream<addLoadingOverlay> onAddLoadingOverlay({required int id}) {
    throw UnimplementedError('onAddLoadingOverlay() has not been implemented.');
  }

  Stream<updateLoadingProgress> onUpdateLoadingProgress(
      {required int id, double? progress}) {
    throw UnimplementedError(
        'onUpdateLoadingProgress() has not been implemented.');
  }

  Stream<throwException> onThrowException(
      {required int id, String? message, int? errorCode}) {
    throw UnimplementedError('onThrowException() has not been implemented.');
  }

  Stream<needInstallARCoreService> onNeedInstallARCoreService(
      {required int id}) {
    throw UnimplementedError(
        'onNeedInstallARCoreService() has not been implemented.');
  }

  Stream<sdkInitialized> onSdkInitialized({required int id}) {
    throw UnimplementedError('onSdkInitialized() has not been implemented.');
  }

  Stream<openBuiltInBrowser> onOpenBuiltInBrowser(
      {required int id, String? url}) {
    throw UnimplementedError('openBuiltInBrowser() has not been implemented.');
  }

  Stream<recognitionStart> onRecognitionStart({required int id}) {
    throw UnimplementedError('onRecognitionStart() has not been implemented.');
  }

  Stream<recognitionComplete> onRecognitionComplete({required int id}) {
    throw UnimplementedError(
        'onRecognitionComplete() has not been implemented.');
  }

  Stream<tryAcquireInformation> onTryAcquireInformation(
      {required int id, String? opTag}) {
    throw UnimplementedError(
        'onTryAcquireInformation() has not been implemented.');
  }

  Stream<packageSizeMoreThanPresetSize> onPackageSizeMoreThanPresetSize(
      {required int id, String? currentSize, String? presetSize}) {
    throw UnimplementedError(
        'onPackageSizeMoreThanPresetSize() has not been implemented.');
  }

  Stream<onARMODExit> onOnARMODExit({required int id}) {
    throw UnimplementedError('onARMODExit() has not been implemented.');
  }

  Stream<onARMODLaunch> onOnARMODLaunch({required int id}) {
    throw UnimplementedError('onARMODLaunch() has not been implemented.');
  }

  /// Dispose of whatever resources the `id` is holding on to.
  void dispose({required int id}) {
    throw UnimplementedError('dispose() has not been implemented.');
  }

  /// Returns a widget displaying the map view
  Widget buildView(
      Map<String, dynamic> creationParams,
      Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
      PlatformViewCreatedCallback onPlatformViewCreated,
      bool useAndroidView) {
    throw UnimplementedError('buildView() has not been implemented.');
  }
}
