part of flutter_armod_widget;

typedef void ARMODCreatedCallback(ARMODWidgetController controller);

class ARMODWidgetController {
  final _ARMODWidgetState _armodWidgetState;

  /// The armodId for this controller
  final int armodId;
  bool willbeDisposed = false;

  /// Used for cancel the subscription
  StreamSubscription? _onDeviceNotSupport,
      _onRemoveLoadingOverlay,
      _onAddLoadingOverlay,
      _onUpdateLoadingProgress,
      _onThrowException,
      _onNeedInstallARCoreService,
      _onSdkInitialized,
      _onOpenBuiltInBrowser,
      _onRecognitionStart,
      _onRecognitionComplete,
      _onTryAcquireInformation,
      _onPackageSizeMoreThanPresetSize,
      _onARMODExit,
      _onARMODLaunch;

  ARMODWidgetController._(this._armodWidgetState, {required this.armodId}) {
    _connectStreams(armodId);
  }

  /// Initialize [ARMODWidgetController] with [id]
  /// Mainly for internal use when instantiating a [ARMODWidgetController] passed
  /// in [ARMODWidget.onARMODCreated] callback.
  static Future<ARMODWidgetController> init(
      int id, _ARMODWidgetState armodWidgetState) async {
    await ARMODViewFlutterPlatform.instance.init(id);
    return ARMODWidgetController._(
      armodWidgetState,
      armodId: id,
    );
  }

  @visibleForTesting
  MethodChannel? get channel {
    if (ARMODViewFlutterPlatform.instance is MethodChannelARMODViewFlutter) {
      return (ARMODViewFlutterPlatform.instance
              as MethodChannelARMODViewFlutter)
          .channel(armodId);
    }
    return null;
  }

  void _connectStreams(int armodId) {
    if (_armodWidgetState.widget.onDeviceNotSupport != null) {
      _onDeviceNotSupport = ARMODViewFlutterPlatform.instance
          .onDeviceNotSupport(id: armodId)
          .listen(
              (ARMODEvent e) => _armodWidgetState.widget.onDeviceNotSupport!());
    }

    if (_armodWidgetState.widget.onRemoveLoadingOverlay != null) {
      _onRemoveLoadingOverlay = ARMODViewFlutterPlatform.instance
          .onRemoveLoadingOverlay(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onRemoveLoadingOverlay!());
    }

    if (_armodWidgetState.widget.onAddLoadingOverlay != null) {
      _onAddLoadingOverlay = ARMODViewFlutterPlatform.instance
          .onAddLoadingOverlay(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onAddLoadingOverlay!());
    }

    if (_armodWidgetState.widget.onUpdateLoadingProgress != null) {
      _onUpdateLoadingProgress = ARMODViewFlutterPlatform.instance
          .onUpdateLoadingProgress(id: armodId)
          .listen((ARMODEvent e) => {
                _armodWidgetState
                    .widget.onUpdateLoadingProgress!(e.value1['progress']),
              });
    }

    if (_armodWidgetState.widget.onThrowException != null) {
      _onThrowException = ARMODViewFlutterPlatform.instance
          .onThrowException(id: armodId)
          .listen((ARMODEvent e) => _armodWidgetState.widget.onThrowException!(
              e.value1['errorMsg'], e.value1['errorCode']));
    }

    if (_armodWidgetState.widget.onNeedInstallARCoreService != null) {
      _onNeedInstallARCoreService = ARMODViewFlutterPlatform.instance
          .onNeedInstallARCoreService(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onNeedInstallARCoreService!());
    }

    if (_armodWidgetState.widget.onSdkInitialized != null) {
      _onSdkInitialized = ARMODViewFlutterPlatform.instance
          .onSdkInitialized(id: armodId)
          .listen(
              (ARMODEvent e) => _armodWidgetState.widget.onSdkInitialized!());
    }

    if (_armodWidgetState.widget.onOpenBuiltInBrowser != null) {
      _onOpenBuiltInBrowser = ARMODViewFlutterPlatform.instance
          .onOpenBuiltInBrowser(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onOpenBuiltInBrowser!(e.value1['url']));
    }

    if (_armodWidgetState.widget.onRecognitionStart != null) {
      _onRecognitionStart = ARMODViewFlutterPlatform.instance
          .onRecognitionStart(id: armodId)
          .listen(
              (ARMODEvent e) => _armodWidgetState.widget.onRecognitionStart!());
    }

    if (_armodWidgetState.widget.onRecognitionComplete != null) {
      _onRecognitionComplete = ARMODViewFlutterPlatform.instance
          .onRecognitionComplete(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onRecognitionComplete!());
    }

    if (_armodWidgetState.widget.onTryAcquireInformation != null) {
      String result = "";
      _onTryAcquireInformation = ARMODViewFlutterPlatform.instance
          .onTryAcquireInformation(id: armodId)
          .listen((tryAcquireInformation e) async => {
                result = await _armodWidgetState
                    .widget.onTryAcquireInformation!(e.value1['opTag']),
                if (!willbeDisposed) e.callback(result),
              });
    }

    if (_armodWidgetState.widget.onPackageSizeMoreThanPresetSize != null) {
      _onPackageSizeMoreThanPresetSize = ARMODViewFlutterPlatform.instance
          .onPackageSizeMoreThanPresetSize(id: armodId)
          .listen((ARMODEvent e) =>
              _armodWidgetState.widget.onPackageSizeMoreThanPresetSize!(
                  e.value1['currentSize'], e.value1['presetSize']));
    }

    if (_armodWidgetState.widget.onARMODExit != null) {
      _onARMODExit = ARMODViewFlutterPlatform.instance
          .onOnARMODExit(id: armodId)
          .listen((ARMODEvent e) => _armodWidgetState.widget.onARMODExit!());
    }

    if (_armodWidgetState.widget.onARMODLaunch != null) {
      _onARMODLaunch = ARMODViewFlutterPlatform.instance
          .onOnARMODLaunch(id: armodId)
          .listen((ARMODEvent e) => _armodWidgetState.widget.onARMODLaunch!());
    }
  }

  Future<String> tryAcquireInformationListener(String opTag) async {
    return await _armodWidgetState.widget.onTryAcquireInformation!(opTag);
  }

  /// Checks to see if armod player is ready to be used
  /// Returns `true` if armod player is ready.
  Future<bool?>? isReady() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.isReady(id: armodId);
    }
    return null;
  }

  /// Get the current pause state of the armod player
  /// Returns `true` if armod player is paused.
  Future<bool?>? isPaused() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.isPaused(id: armodId);
    }
    return null;
  }

  /// Get the current load state of the armod player
  /// Returns `true` if armod player is loaded.
  Future<bool?>? isLoaded() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.isLoaded(id: armodId);
    }
    return null;
  }

  /// Helper method to know if armod has been put in background mode (WIP) unstable
  /// Returns `true` if armod player is in background.
  Future<bool?>? inBackground() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.inBackground(id: armodId);
    }
    return null;
  }

  /// Creates a armod player if it's not already created. Please only call this if armod is not ready,
  /// or is in unloaded state. Use [isLoaded] to check.
  /// Returns `true` if armod player was created succesfully.
  Future<bool?>? create() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.createARMODPlayer(id: armodId);
    }
    return null;
  }

  /// Creates a armod player if it's not already created. Please only call this if armod is not ready,
  /// or is in unloaded state. Use [isLoaded] to check.
  /// Returns `true` if armod player was created succesfully.
  Future<bool?>? closeARMODPlayer() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.closeARMODPlayer(id: armodId);
    }
    return null;
  }

  /// Post message to armod from flutter. This method takes in a string [message].
  /// The [gameObject] must match the name of an actual armod game object in a scene at runtime, and the [methodName],
  /// must exist in a `MonoDevelop` `class` and also exposed as a method. [message] is an parameter taken by the method
  ///
  /// ```dart
  /// postMessage("GameManager", "openScene", "ThirdScene")
  /// ```
  Future<void>? postMessage(String gameObject, methodName, message) {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.postMessage(
        id: armodId,
        gameObject: gameObject,
        methodName: methodName,
        message: message,
      );
    }
    return null;
  }

  ///Initialize ARMOD SDK with configuration information. This method will call InitARMODModule automatically.
  Future<void>? initARMOD(String _appConfigure) {
    postMessage("EntryPoint", "InitSDK", _appConfigure);
  }

  ///Query the ARExperience projecct by project uid.
  Future<void>? fetchProject(String _projectUid) {
    postMessage("EntryPoint", "LaunchARQuery", _projectUid);
  }

  ///Query the ARExperience projecct by image recognized
  Future<void>? fetchProjectByImage() {
    postMessage("EntryPoint", "LaunchARQuery", "");
  }

  ///Clean ARExperiences cache in the device. Only AR-MOD in woking!
  Future<void>? cleanCache() {
    postMessage("EntryPoint", "cleanCache", "");
  }

  ///Unload current ARMOD Window
  Future<void>? unloadAndHideARMOD() {
    // postMessage("EntryPoint", "Dispose", "");
    ARMODViewFlutterPlatform.instance.closeARMODPlayer(id: armodId);
    willbeDisposed = true;
  }

  Future<void>? setDeivcesOrientation(String _orientation) {
    ARMODViewFlutterPlatform.instance
        .setDevicesOrientation(id: armodId, orientation: _orientation);
  }

  /// Pause the armod in-game player with this method
  Future<void>? pause() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.pausePlayer(id: armodId);
    }
    return null;
  }

  /// Resume the armod in-game player with this method idf it is in a paused state
  Future<void>? resume() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.resumePlayer(id: armodId);
    }
    return null;
  }

  /// Sometimes you want to open armod in it's own process and openInNativeProcess does just that.
  /// It works for Android and iOS is WIP
  Future<void>? openInNativeProcess() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.openInNativeProcess(id: armodId);
    }
    return null;
  }

  /// Unloads armod player from th current process (Works on Android only for now)
  /// For more information please read [Unity Docs](https://docs.unity3d.com/2020.2/Documentation/Manual/UnityasaLibrary.html)
  Future<void>? unload() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.unloadPlayer(id: armodId);
    }
    return null;
  }

  /// Quits armod player. Note that this kills the current flutter process, thus quiting the app
  Future<void>? quit() {
    if (!_armodWidgetState.widget.enablePlaceholder) {
      return ARMODViewFlutterPlatform.instance.quitPlayer(id: armodId);
    }
    return null;
  }

  /// Cancel the subscriptions when dispose called
  void _cancelSubscriptions() {
    _onDeviceNotSupport?.cancel();
    _onRemoveLoadingOverlay?.cancel();
    _onAddLoadingOverlay?.cancel();
    _onUpdateLoadingProgress?.cancel();
    _onThrowException?.cancel();
    _onNeedInstallARCoreService?.cancel();
    _onSdkInitialized?.cancel();
    _onOpenBuiltInBrowser?.cancel();
    _onRecognitionStart?.cancel();
    _onRecognitionComplete?.cancel();
    _onTryAcquireInformation?.cancel();
    _onPackageSizeMoreThanPresetSize?.cancel();
    _onARMODExit?.cancel();
    _onARMODLaunch?.cancel();

    _onDeviceNotSupport = null;
    _onRemoveLoadingOverlay = null;
    _onAddLoadingOverlay = null;
    _onUpdateLoadingProgress = null;
    _onThrowException = null;
    _onNeedInstallARCoreService = null;
    _onSdkInitialized = null;
    _onOpenBuiltInBrowser = null;
    _onRecognitionStart = null;
    _onRecognitionComplete = null;
    _onTryAcquireInformation = null;
    _onPackageSizeMoreThanPresetSize = null;
    _onARMODExit = null;
    _onARMODLaunch = null;
  }

  void dispose() {
    Future.delayed(Duration(milliseconds: 250), () {
      _cancelSubscriptions();
      ARMODViewFlutterPlatform.instance.dispose(id: armodId);
    });
  }
}

typedef void DeviceNotSupportCallback();

typedef void AddLoadingOverlayCallback();

typedef void RemoveLoadingOverlayCallback();

typedef void UpdateLoadingProgressCallback(double progress);

typedef void ThrowExceptionCallback(String message, int errorCode);

typedef void NeedInstallARCoreServiceCallback();

typedef void SdkInitializedCallback();

typedef void OpenBuiltInBrowserCallback(String url);

typedef void RecognitionStartCallback();

typedef void RecognitionCompleteCallback();

typedef Future<String> TryAcquireInformationCallback(String opTag);

typedef void PackageSizeMoreThanPresetSizeCallback(
    String version1, String version2);

typedef void ARMODExitCallback();

typedef void ARMODLaunchCallback();
